module model_W25Q16CV #(
    parameter       TCO = 6.21 // Represents board delays plus Tco of part
)(
    input           CS,
    input           CLK,
    inout   [3:0]   IO
);
    timeunit 1ns;
    timeprecision 100ps;

    reg continuous_read = '1;   //pretend we started up in the right state
    reg is_out = '0;

    reg shift_out [0:3];

    assign IO[0] = is_out ? shift_out[0] : 'Z;
    assign IO[1] = is_out ? shift_out[1] : 'Z;
    assign IO[2] = is_out ? shift_out[2] : 'Z;
    assign IO[3] = is_out ? shift_out[3] : 'Z;

    reg [7:0] long_shift_in [0:3];
    wire [31:0] address_mbits;

    assign address_mbits = {
        long_shift_in[3][7], long_shift_in[2][7], long_shift_in[1][7], long_shift_in[0][7],
        long_shift_in[3][6], long_shift_in[2][6], long_shift_in[1][6], long_shift_in[0][6],
        long_shift_in[3][5], long_shift_in[2][5], long_shift_in[1][5], long_shift_in[0][5],
        long_shift_in[3][4], long_shift_in[2][4], long_shift_in[1][4], long_shift_in[0][4],
        long_shift_in[3][3], long_shift_in[2][3], long_shift_in[1][3], long_shift_in[0][3],
        long_shift_in[3][2], long_shift_in[2][2], long_shift_in[1][2], long_shift_in[0][2],
        long_shift_in[3][1], long_shift_in[2][1], long_shift_in[1][1], long_shift_in[0][1],
        long_shift_in[3][0], long_shift_in[2][0], long_shift_in[1][0], long_shift_in[0][0]};

    reg [23:0] address;

    // Populate the first 8192 bits (arbitrary number) with a pattern 1, 2, 3...
    reg [7:0] mem_data [0:1023];
    initial begin
        for(integer n=0; n<1024; n++) begin
            mem_data[n] = n[7:0] + 1'd1;
        end
    end

    always @(negedge CS) begin
        is_out = '0;
        long_shift_in[0] = '0;
        long_shift_in[1] = '0;
        long_shift_in[2] = '0;
        long_shift_in[3] = '0;
        fork
            begin
                if(continuous_read) begin
                    endless_quad_read;
                end else begin
                    repeat(8) begin
                        @(posedge CLK);
                        long_shift_in[0] = {long_shift_in[0][6:0], IO[0]};
                    end
                    if(long_shift_in[0]==8'hEB) begin   //Fast Quad IO Read
                        endless_quad_read;
                    end
                end
            end
            begin
                @(posedge CS);
            end
        join_any
        disable fork;
        is_out = '0;
    end

    task endless_quad_read;
        repeat(8) begin
            @(posedge CLK);
            long_shift_in[0] = {long_shift_in[0][6:0], IO[0]};
            long_shift_in[1] = {long_shift_in[1][6:0], IO[1]};
            long_shift_in[2] = {long_shift_in[2][6:0], IO[2]};
            long_shift_in[3] = {long_shift_in[3][6:0], IO[3]};
        end
        #1;
        continuous_read = address_mbits[5:4]==2'b10;
        address = address_mbits[31:8];
        repeat(4) @(posedge CLK);   //Dummy cycles
        forever begin
            @(negedge CLK) begin
                is_out <= #TCO '1;
                shift_out[0] <= #TCO mem_data[address][4];
                shift_out[1] <= #TCO mem_data[address][5];
                shift_out[2] <= #TCO mem_data[address][6];
                shift_out[3] <= #TCO mem_data[address][7];
            end
            @(negedge CLK) begin
                shift_out[0] <= #TCO mem_data[address][0];
                shift_out[1] <= #TCO mem_data[address][1];
                shift_out[2] <= #TCO mem_data[address][2];
                shift_out[3] <= #TCO mem_data[address][3];
                address <= address + 1'd1;
            end
        end
    endtask

endmodule