module clock_gen #(
	parameter			frequency = 50e6
)
(
	output	bit			clock_out = '0
);
	timeunit 1ns;
	timeprecision 100ps;
	
	real real_freq = frequency;
	real half_cycle_time;
	bit running = 1;
	
	always_comb half_cycle_time = ((1e9)/frequency)/2;

	initial begin
		forever begin
			fork
				begin: clock_toggling
					forever clock_out = #half_cycle_time ~clock_out;
				end
				begin: check_for_stop
					while(running) #1;
				end
			join_any
			disable fork;
			#1;
		end	
	end
	
	task stop;
		begin
			running = 0;
		end
	endtask
	
	task start;
		begin
			running = 1;
		end
	endtask
	
	task set_frequency(
		input	real real_new_freq
	);
		real_freq = real_new_freq;
	endtask
	
endmodule
