`timescale 1ns/100ps

module  tb_silo;
    
    wire            CLK24M;
    
    wire    [3:0]   FLASH_IO;
    wire            FLASH_SCLK;
    wire            FLASH_CS;
    
    wire    [7:0]   FLASH_BYTE;
    wire            FLASH_VALID;
    
    clock_gen #(.frequency(24e6)) m_clock_fast(.clock_out(CLK24M));
    
    fpga_silo m_silo(
        .*
    );
    
    model_W25Q16CV m_flash(
        .CS         (FLASH_CS),
        .CLK        (FLASH_SCLK),
        .IO         (FLASH_IO)
    );
    
    // Simple check of data coming in. Should be a pattern of 1, 2, 3... 8, 1, 2, 3... 8.
    // That pattern is currently hard-coded into the flash memory model.
    logic [7:0] byte_expected = 8'd1;
    always @(posedge FLASH_SCLK) begin
        if(FLASH_VALID) begin
            if(FLASH_BYTE !== byte_expected) begin
                $display("\nERROR! Wrong value detected!");
                $display("Expected %8b", byte_expected);
                $display("Saw      %8b", FLASH_BYTE);
                $stop;
            end
            byte_expected++;
            if(byte_expected > 8'd8) byte_expected = 8'd1;
        end
    end
    
    initial begin
        repeat(100) @(posedge FLASH_VALID);
        $display("\nDONE\nNo errors detected\n");
        $stop;
    end
endmodule
